@@ -2,7 +2,7 @@
 
 import os
 import sys
-from typing import List, Dict, Any
+from typing import List, Dict, Any, Optional
 from pydantic_ai import Agent, RunContext
 from pydantic_ai.models.anthropic import AnthropicModel
 from .utils import (
@@ -13,6 +13,7 @@ from .utils import (
     is_blank_project,
     extract_code_blocks
 )
+from .config import WYN360Config
 
 
 class WYN360Agent:
@@ -23,7 +24,13 @@ class WYN360Agent:
     using Anthropic Claude via pydantic_ai.
     """
 
-    def __init__(self, api_key: str, model: str = "claude-sonnet-4-20250514", use_history: bool = True):
+    def __init__(
+        self,
+        api_key: str,
+        model: str = "claude-sonnet-4-20250514",
+        use_history: bool = True,
+        config: Optional[WYN360Config] = None
+    ):
         """
         Initialize the WYN360 Agent.
 
@@ -31,9 +38,17 @@ class WYN360Agent:
             api_key: Anthropic API key
             model: Claude model to use (default: claude-sonnet-4-20250514)
             use_history: Whether to send conversation history with each request (default: True)
+            config: Optional WYN360Config object with user/project configuration
         """
         self.api_key = api_key
-        self.model_name = model
+        self.config = config
+
+        # Use config model if available, otherwise use provided model
+        if config:
+            self.model_name = config.model
+        else:
+            self.model_name = model
+
         self.use_history = use_history
         self.conversation_history: List[Dict[str, str]] = []
 
@@ -47,7 +62,7 @@ class WYN360Agent:
         os.environ['ANTHROPIC_API_KEY'] = api_key
 
         # Initialize Anthropic model (it will use the environment variable)
-        self.model = AnthropicModel(model)
+        self.model = AnthropicModel(self.model_name)
 
         # Create the agent with tools
         self.agent = Agent(
@@ -68,12 +83,12 @@ class WYN360Agent:
                 self.move_file,
                 self.create_directory
             ],
-            retries=3  # Allow up to 3 retries for tool calls
+            retries=0  # No retries - show errors immediately to model for correction
         )
 
     def _get_system_prompt(self) -> str:
         """Get the system prompt for the coding assistant."""
-        return """You are WYN360, an intelligent AI coding assistant. Your role is to help users with:
+        base_prompt = """You are WYN360, an intelligent AI coding assistant. Your role is to help users with:
 
 1. **Starting new projects**: Generate well-structured Python code from scratch
 2. **Improving existing projects**: Analyze codebases and suggest/implement improvements
@@ -98,15 +113,21 @@ Guidelines:
 - Asks to "write code to analyze", "write code to explore"
 
 ACTION:
+- **KEEP CODE CONCISE** - Write minimal, focused code under 100KB (approximately 800-1000 lines MAX)
+- For EDA/analysis scripts: include only essential imports, loading, and 3-5 key visualizations
+- For ML models: ONE model type, essential preprocessing, train/evaluate, save model - NO extensive hyperparameter tuning
+- If script would be >100KB, break into multiple smaller files (e.g., train_model.py, evaluate_model.py)
+- **CRITICAL**: Exceeding size limits causes "exceeded max retries" errors - ALWAYS keep code minimal
 - Use write_file with overwrite=False (default)
-- If write_file fails with "already exists" error, automatically retry with overwrite=True
+- If write_file returns an "already exists" error message, call write_file ONCE MORE with overwrite=True
 - Do NOT read_file first (the file doesn't exist yet)
 - Suggest a descriptive filename if user doesn't specify one
+- If tool fails with "exceeded max retries count of 0", the content is likely too large - reduce code size
 
 EXAMPLES:
 - "write a .py script to analyze data.csv" → Create analyze_data.py with overwrite=False
 - "generate a script for data exploration" → Create data_exploration.py with overwrite=False
-- If it fails (file exists), immediately retry: write_file with overwrite=True
+- If result is "already exists", then: write_file with overwrite=True ONCE
 
 **UPDATING EXISTING FILES** - When user says:
 - "add feature", "update", "improve", "modify", "change"
@@ -164,6 +185,26 @@ Notes:
 - Always preserve the success/failure indicator from tool output
 """
 
+        # Add custom instructions from config if available
+        if self.config:
+            if self.config.custom_instructions:
+                base_prompt += "\n\n**Custom Instructions:**\n"
+                base_prompt += self.config.custom_instructions
+
+            # Add project context if available
+            if self.config.project_context:
+                base_prompt += "\n\n**Project Context:**\n"
+                base_prompt += self.config.project_context
+
+            # Add project dependencies info if available
+            if self.config.project_dependencies:
+                base_prompt += "\n\n**Project Dependencies:**\n"
+                base_prompt += "This project uses the following key dependencies:\n"
+                for dep in self.config.project_dependencies:
+                    base_prompt += f"- {dep}\n"
+
+        return base_prompt
+
     async def read_file(self, ctx: RunContext[None], file_path: str) -> str:
         """
         Read the contents of a file.
@@ -183,23 +224,81 @@ Notes:
     async def write_file(
         self,
         ctx: RunContext[None],
-        file_path: str,
-        content: str,
+        file_path,  # Accept any type, validate manually
+        content,    # Accept any type, validate manually
         overwrite: bool = False
     ) -> str:
         """
         Write content to a file.
 
+        IMPORTANT: Keep code concise and under 50KB. For large scripts, break into smaller files.
+
         Args:
-            file_path: Path where to write the file
-            content: Content to write
-            overwrite: Whether to overwrite if file exists
+            file_path: Path where to write the file (string)
+            content: Content to write (string) - MUST be under 1MB
+            overwrite: Whether to overwrite if file exists (boolean, default False)
 
         Returns:
             Success or error message
+
+        Note: This tool has NO type validation at the framework level to prevent parameter errors.
+        All validation happens inside the function with clear error messages.
         """
-        success, message = write_file_safe(file_path, content, overwrite)
-        return message
+        try:
+            # First, check if content is even a valid type
+            if content is None:
+                return "Error: content parameter is None. You must provide code content to write."
+
+            # Try to get length - if this fails, content is wrong type
+            try:
+                content_length = len(content)
+            except TypeError as e:
+                return f"Error: content must be a string or have length, got type {type(content).__name__}: {str(e)}"
+
+            # Now we know content has a length, check if it's a string
+            if not isinstance(content, str):
+                # Try to convert to string
+                try:
+                    content = str(content)
+                    content_length = len(content)
+                except Exception as e:
+                    return f"Error: Cannot convert content to string. Type: {type(content).__name__}, Error: {str(e)}"
+
+            # Validate file_path
+            if not file_path:
+                return "Error: file_path is required and cannot be empty."
+
+            if not isinstance(file_path, str):
+                try:
+                    file_path = str(file_path)
+                except Exception as e:
+                    return f"Error: Cannot convert file_path to string. Type: {type(file_path).__name__}, Error: {str(e)}"
+
+            # Validate content size EARLY (prevent extremely large files)
+            max_size = 100_000  # Reduced to 100KB from 1MB
+            if content_length > max_size:
+                # Truncate and show preview
+                preview = content[:500] + f"\n\n... ({content_length - 500} more bytes) ..."
+                return f"Error: Content too large ({content_length} bytes, {content_length // 1024}KB). Maximum size is {max_size} bytes ({max_size // 1024}KB).\n\nYour code is too long! Please reduce to under 1000 lines. Break into smaller files if needed.\n\nContent preview:\n{preview}"
+
+            content_preview = content[:100] if content_length > 100 else content
+
+            # Try to write the file
+            success, message = write_file_safe(file_path, content, overwrite)
+
+            # If file exists and overwrite is False, provide clear guidance
+            if not success and "already exists" in message:
+                return f"{message}\n\nNote: If you want to update this file, you must explicitly set overwrite=True in your next write_file call."
+
+            return message
+        except TypeError as e:
+            return f"TypeError in write_file: {str(e)}. file_path type: {type(file_path)}, content type: {type(content)}, overwrite type: {type(overwrite)}"
+        except Exception as e:
+            # Log the actual exception for debugging
+            import traceback
+            tb = traceback.format_exc()
+            error_msg = f"Unexpected error in write_file: {type(e).__name__}: {str(e)}\n\nTraceback:\n{tb}"
+            return error_msg
 
     async def list_files(self, ctx: RunContext[None], directory: str = ".") -> str:
         """
@@ -283,9 +382,14 @@ Notes:
             sys.stdout.flush()  # Force output to appear immediately
 
             response = input("Execute this command? (y/N): ").strip().lower()
-            print()  # Add spacing after response
 
-            if response not in ['y', 'yes']:
+            # Confirm user's input
+            if response in ['y', 'yes']:
+                print("✓ Confirmed. Executing command...\n")
+                sys.stdout.flush()
+            else:
+                print(f"✗ Cancelled (pressed '{response or 'N'}').\n")
+                sys.stdout.flush()
                 return "❌ Command execution cancelled by user."
 
         success, output, return_code = execute_command_safe(command, timeout)
@@ -583,6 +687,67 @@ Notes:
             error_msg = f"An error occurred: {str(e)}"
             return error_msg
 
+    async def chat_stream(self, user_message: str):
+        """
+        Process a user message and return complete response.
+        CLI will simulate streaming by printing word-by-word.
+
+        This uses pydantic-ai's run() which executes tools properly.
+
+        Args:
+            user_message: The user's input message
+
+        Returns:
+            Complete response text from the agent
+        """
+        try:
+            # Add user message to history
+            self.conversation_history.append({
+                "role": "user",
+                "content": user_message
+            })
+
+            # Use run() to get complete response (not run_stream)
+            # This ensures tools execute properly
+            result = await self.agent.run(user_message)
+
+            # Extract response text
+            response_text = getattr(result, 'data', None) or getattr(result, 'output', str(result))
+            if not isinstance(response_text, str):
+                response_text = str(response_text)
+
+            # Add assistant response to history
+            self.conversation_history.append({
+                "role": "assistant",
+                "content": response_text
+            })
+
+            # Track token usage
+            self._track_tokens(user_message, response_text)
+
+            # Check if response contains code blocks and extract them
+            code_blocks = extract_code_blocks(response_text)
+
+            # If there are Python code blocks, offer to save them
+            if code_blocks:
+                python_blocks = [b for b in code_blocks if b['language'] == 'python']
+                if python_blocks and is_blank_project():
+                    # Auto-save first Python code block in blank projects
+                    code = python_blocks[0]['code']
+                    # Try to determine filename from code or use default
+                    filename = self._suggest_filename(code)
+                    success, msg = write_file_safe(filename, code, overwrite=False)
+                    if success:
+                        save_message = f"\n\n✓ Code saved to: {filename}"
+                        response_text = response_text + save_message
+
+            # Return complete response
+            return response_text
+
+        except Exception as e:
+            error_msg = f"\n\nAn error occurred: {str(e)}"
+            return error_msg
+
     def _suggest_filename(self, code: str) -> str:
         """
         Suggest a filename based on code content.
